@@ -183,65 +183,58 @@ int do_command(struct local_client *client, struct clvm_header *msg, int msglen,
 	}
 
 	return status;
-
 }
 
 static int lock_vg(struct local_client *client)
 {
-    struct dm_hash_table *lock_hash;
-    struct clvm_header *header =
-	(struct clvm_header *) client->bits.localsock.cmd;
-    unsigned char lock_cmd;
-    int lock_mode;
-    char *args = header->node + strlen(header->node) + 1;
-    int lkid;
-    int status = 0;
-    char *lockname;
+	struct dm_hash_table *lock_hash;
+	struct clvm_header *header =
+		(struct clvm_header *) client->bits.localsock.cmd;
+	unsigned char lock_cmd;
+	int lock_mode;
+	char *args = header->node + strlen(header->node) + 1;
+	int lkid;
+	int status;
+	char *lockname;
 
-    /* Keep a track of VG locks in our own hash table. In current
-       practice there should only ever be more than two VGs locked
-       if a user tries to merge lots of them at once */
-    if (client->bits.localsock.private) {
-	lock_hash = (struct dm_hash_table *)client->bits.localsock.private;
-    }
-    else {
-	lock_hash = dm_hash_create(3);
-	if (!lock_hash)
-	    return ENOMEM;
-	client->bits.localsock.private = (void *)lock_hash;
-    }
+	/*
+	 * Keep a track of VG locks in our own hash table. In current
+	 * practice there should only ever be more than two VGs locked
+	 * if a user tries to merge lots of them at once
+	 */
+	if (!client->bits.localsock.private) {
+		if (!(lock_hash = dm_hash_create(3)))
+			return ENOMEM;
+		client->bits.localsock.private = (void *) lock_hash;
+	} else
+		lock_hash = (struct dm_hash_table *) client->bits.localsock.private;
 
-    lock_cmd = args[0] & (LCK_NONBLOCK | LCK_HOLD | LCK_SCOPE_MASK | LCK_TYPE_MASK);
-    lock_mode = ((int)lock_cmd & LCK_TYPE_MASK);
-    /* lock_flags = args[1]; */
-    lockname = &args[2];
-    DEBUGLOG("doing PRE command LOCK_VG '%s' at %x (client=%p)\n", lockname, lock_cmd, client);
+	lock_cmd = args[0] & (LCK_NONBLOCK | LCK_HOLD | LCK_SCOPE_MASK | LCK_TYPE_MASK);
+	lock_mode = ((int) lock_cmd & LCK_TYPE_MASK);
+	/* lock_flags = args[1]; */
+	lockname = &args[2];
+	DEBUGLOG("doing PRE command LOCK_VG '%s' at %x (client=%p)\n", lockname, lock_cmd, client);
 
-    if (lock_mode == LCK_UNLOCK) {
+	if (lock_mode == LCK_UNLOCK) {
+		if (!(lkid = (int) (long) dm_hash_lookup(lock_hash, lockname)))
+			return EINVAL;
 
-	lkid = (int)(long)dm_hash_lookup(lock_hash, lockname);
-	if (lkid == 0)
-	    return EINVAL;
+		if ((status = sync_unlock(lockname, lkid)))
+			status = errno;
+		else
+			dm_hash_remove(lock_hash, lockname);
+	} else {
+		/* Read locks need to be PR; other modes get passed through */
+		if (lock_mode == LCK_READ)
+			lock_mode = LCK_PREAD;
 
-	status = sync_unlock(lockname, lkid);
-	if (status)
-	    status = errno;
-	else
-	    dm_hash_remove(lock_hash, lockname);
-    }
-    else {
-	/* Read locks need to be PR; other modes get passed through */
-	if (lock_mode == LCK_READ)
-	    lock_mode = LCK_PREAD;
-	status = sync_lock(lockname, lock_mode, (lock_cmd & LCK_NONBLOCK) ? LCKF_NOQUEUE : 0, &lkid);
-	if (status)
-	    status = errno;
-	else
-	    if (!dm_hash_insert(lock_hash, lockname, (void *)(long)lkid))
-                    return ENOMEM;
-    }
+		if ((status = sync_lock(lockname, lock_mode, (lock_cmd & LCK_NONBLOCK) ? LCKF_NOQUEUE : 0, &lkid)))
+			status = errno;
+		else if (!dm_hash_insert(lock_hash, lockname, (void *) (long) lkid))
+			return ENOMEM;
+	}
 
-    return status;
+	return status;
 }
 
 
@@ -314,9 +307,8 @@ int do_post_command(struct local_client *client)
 	init_test(0);
 	switch (header->cmd) {
 	case CLVMD_CMD_TEST:
-		status =
-		    sync_unlock("CLVMD_TEST", (int) (long) client->bits.localsock.private);
-		client->bits.localsock.private = 0;
+		status = sync_unlock("CLVMD_TEST", (int) (long) client->bits.localsock.private);
+		client->bits.localsock.private = NULL;
 		break;
 
 	case CLVMD_CMD_LOCK_LV:
@@ -357,7 +349,7 @@ void cmd_client_cleanup(struct local_client *client)
 	}
 
 	dm_hash_destroy(lock_hash);
-	client->bits.localsock.private = 0;
+	client->bits.localsock.private = NULL;
 }
 
 
