@@ -109,6 +109,7 @@ static int _init_cluster(void)
 	r = init_comms();
 	if (r) {
 		dm_hash_destroy(_locks);
+		_locks = NULL;
 		return r;
 	}
 
@@ -120,8 +121,6 @@ static void _cluster_closedown(void)
 {
 	close_comms();
 
-	DEBUGLOG("cluster_closedown\n");
-	destroy_lvhash();
 	/* If there is any awaited resource, kill it softly */
 	pthread_mutex_lock(&_lock_mutex);
 	dm_hash_destroy(_locks);
