@@ -180,8 +180,19 @@ static int insert_info(const char *resource, struct lv_info *lvi)
 
 static void remove_info(const char *resource)
 {
+	int num_open;
+
 	pthread_mutex_lock(&lv_hash_lock);
 	dm_hash_remove(lv_hash, resource);
+
+	/* When last lock is remove, validate there are not left opened devices */
+	if (!dm_hash_get_first(lv_hash)) {
+		if (critical_section())
+			log_error(INTERNAL_ERROR "No volumes are locked however clvmd is in activation mode critical section.");
+		if ((num_open = dev_cache_check_for_open_devices()))
+			log_error(INTERNAL_ERROR "No volumes are locked however %d devices are still open.", num_open);
+	}
+
 	pthread_mutex_unlock(&lv_hash_lock);
 }
 
@@ -268,16 +279,13 @@ static int hold_lock(char *resource, int mode, int flags)
 	}
 	if (lvi) {
 		/* Already exists - convert it */
-		status =
-		    sync_lock(resource, mode, flags, &lvi->lock_id);
+		status = sync_lock(resource, mode, flags, &lvi->lock_id);
 		saved_errno = errno;
 		if (!status)
 			lvi->lock_mode = mode;
-
-		if (status) {
+		else
 			DEBUGLOG("hold_lock. convert to %d failed: %s\n", mode,
 				 strerror(errno));
-		}
 		errno = saved_errno;
 	} else {
 		lvi = malloc(sizeof(struct lv_info));
@@ -607,9 +615,8 @@ int post_lock_lv(unsigned char command, unsigned char lock_flags,
 	    (command & LCK_CLUSTER_VG)) {
 		int oldmode;
 
-		DEBUGLOG
-		    ("post_lock_lv: resource '%s', cmd = %s, flags = %s\n",
-		     resource, decode_locking_cmd(command), decode_flags(lock_flags));
+		DEBUGLOG("post_lock_lv: resource '%s', cmd = %s, flags = %s\n",
+			 resource, decode_locking_cmd(command), decode_flags(lock_flags));
 
 		/* If the lock state is PW then restore it to what it was */
 		oldmode = get_current_lock(resource);
@@ -917,6 +924,6 @@ void destroy_lvm(void)
 	if (cmd) {
 		memlock_dec_daemon(cmd);
 		destroy_toolcontext(cmd);
+		cmd = NULL;
 	}
-	cmd = NULL;
 }
