@@ -372,14 +372,13 @@ int daemon_talk(struct dm_event_fifos *fifos,
 	if (!_daemon_write(fifos, msg)) {
 		stack;
 		dm_free(msg->data);
-		msg->data = 0;
+		msg->data = NULL;
 		return -EIO;
 	}
 
 	do {
-
 		dm_free(msg->data);
-		msg->data = 0;
+		msg->data = NULL;
 
 		if (!_daemon_read(fifos, msg)) {
 			stack;
@@ -554,8 +553,8 @@ static struct dm_task *_get_device_info(const struct dm_event_handler *dmevh)
 
 	if (!info.exists) {
 		log_error("_get_device_info: %s%s%s%.0d%s%.0d%s%s: device not found",
-			  dmevh->uuid ? : "", 
-			  (!dmevh->uuid && dmevh->dev_name) ? dmevh->dev_name : "", 
+			  dmevh->uuid ? : "",
+			  (!dmevh->uuid && dmevh->dev_name) ? dmevh->dev_name : "",
 			  (!dmevh->uuid && !dmevh->dev_name && dmevh->major > 0) ? "(" : "",
 			  (!dmevh->uuid && !dmevh->dev_name && dmevh->major > 0) ? dmevh->major : 0,
 			  (!dmevh->uuid && !dmevh->dev_name && dmevh->major > 0) ? ":" : "",
@@ -565,7 +564,6 @@ static struct dm_task *_get_device_info(const struct dm_event_handler *dmevh)
 		goto bad;
 	}
 
-		  
 	return dmt;
 
       bad:
@@ -619,6 +617,13 @@ int dm_event_register_handler(const struct dm_event_handler *dmevh)
 
 	uuid = dm_task_get_uuid(dmt);
 
+	if (!strstr(dmevh->dso, "libdevmapper-event-lvm2thin.so") &&
+	    !strstr(dmevh->dso, "libdevmapper-event-lvm2snapshot.so") &&
+	    !strstr(dmevh->dso, "libdevmapper-event-lvm2mirror.so") &&
+	    !strstr(dmevh->dso, "libdevmapper-event-lvm2raid.so"))
+		log_warn("WARNING: %s: dmeventd plugins are deprecated", dmevh->dso);
+
+
 	if ((err = _do_event(DM_EVENT_CMD_REGISTER_FOR_EVENT, dmevh->dmeventd_path, &msg,
 			     dmevh->dso, uuid, dmevh->mask, dmevh->timeout)) < 0) {
 		log_error("%s: event registration failed: %s",
@@ -683,7 +688,7 @@ static char *_fetch_string(char **src, const int delimiter)
 static int _parse_message(struct dm_event_daemon_message *msg, char **dso_name,
 			 char **uuid, enum dm_event_mask *evmask)
 {
-	char *id = NULL;
+	char *id;
 	char *p = msg->data;
 
 	if ((id = _fetch_string(&p, ' ')) &&
@@ -695,8 +700,7 @@ static int _parse_message(struct dm_event_daemon_message *msg, char **dso_name,
 		return 0;
 	}
 
-	if (id)
-		dm_free(id);
+	dm_free(id);
 	return -ENOMEM;
 }
 
@@ -836,7 +840,7 @@ static char *_skip_string(char *src, const int delimiter)
 
 int dm_event_set_timeout(const char *device_path, uint32_t timeout)
 {
-	struct dm_event_daemon_message msg = { 0, 0, NULL };
+	struct dm_event_daemon_message msg = { 0 };
 
 	if (!device_exists(device_path))
 		return -ENODEV;
@@ -848,7 +852,7 @@ int dm_event_set_timeout(const char *device_path, uint32_t timeout)
 int dm_event_get_timeout(const char *device_path, uint32_t *timeout)
 {
 	int ret;
-	struct dm_event_daemon_message msg = { 0, 0, NULL };
+	struct dm_event_daemon_message msg = { 0 };
 
 	if (!device_exists(device_path))
 		return -ENODEV;
@@ -858,12 +862,13 @@ int dm_event_get_timeout(const char *device_path, uint32_t *timeout)
 		if (!p) {
 			log_error("malformed reply from dmeventd '%s'\n",
 				  msg.data);
+			dm_free(msg.data);
 			return -EIO;
 		}
 		*timeout = atoi(p);
 	}
-	if (msg.data)
-		dm_free(msg.data);
+	dm_free(msg.data);
+
 	return ret;
 }
 #endif
