@@ -16,8 +16,6 @@
 #include "log.h"
 
 #include "lvm2cmd.h"
-#include "errors.h"
-#include "libdevmapper-event.h"
 #include "dmeventd_lvm.h"
 
 #include <pthread.h>
@@ -82,10 +80,7 @@ static void _temporary_log_fn(int level,
 
 void dmeventd_lvm2_lock(void)
 {
-	if (pthread_mutex_trylock(&_event_mutex)) {
-		syslog(LOG_NOTICE, "Another thread is handling an event. Waiting...");
-		pthread_mutex_lock(&_event_mutex);
-	}
+	pthread_mutex_lock(&_event_mutex);
 }
 
 void dmeventd_lvm2_unlock(void)
@@ -163,6 +158,11 @@ int dmeventd_lvm2_command(struct dm_pool *mem, char *buffer, size_t size,
 		return 0;
 	}
 
+	/* strip off the mirror component designations */
+	if ((layer = strstr(lv, "_mimagetmp")) ||
+	    (layer = strstr(lv, "_mlog")))
+		*layer = '\0';
+
 	r = dm_snprintf(buffer, size, "%s %s/%s", cmd, vg, lv);
 
 	dm_pool_free(mem, vg);
