@@ -66,7 +66,18 @@
 #define DEFAULT_DMEVENTD_MONITOR 1
 #define DEFAULT_BACKGROUND_POLLING 1
 
-#define DEFAULT_THIN_CHECK_OPTIONS "-q"
+#ifndef DMEVENTD_PATH
+#  define DEFAULT_DMEVENTD_PATH ""
+#else
+#  define DEFAULT_DMEVENTD_PATH DMEVENTD_PATH
+#endif
+
+#ifdef THIN_CHECK_NEEDS_CHECK
+#  define DEFAULT_THIN_CHECK_OPTIONS "-q --clear-needs-check-flag"
+#else
+#  define DEFAULT_THIN_CHECK_OPTIONS "-q"
+#endif
+
 #define DEFAULT_THIN_REPAIR_OPTIONS ""
 #define DEFAULT_THIN_POOL_METADATA_REQUIRE_SEPARATE_PVS 0
 #define DEFAULT_THIN_POOL_MAX_METADATA_SIZE (16 * 1024 * 1024)  /* KB */
@@ -77,7 +88,12 @@
 #define DEFAULT_THIN_POOL_CHUNK_SIZE_PERFORMANCE 512 /* KB */
 #define DEFAULT_THIN_POOL_DISCARDS "passdown"
 #define DEFAULT_THIN_POOL_ZERO 1
-#define DEFAULT_POOL_METADATA_SPARE 1
+#define DEFAULT_POOL_METADATA_SPARE 1 /* thin + cache */
+
+#define DEFAULT_CACHE_POOL_METADATA_REQUIRE_SEPARATE_PVS 0
+#define DEFAULT_CACHE_POOL_CHUNK_SIZE 64 /* KB */
+#define DEFAULT_CACHE_POOL_MIN_METADATA_SIZE 2048  /* KB */
+#define DEFAULT_CACHE_POOL_MAX_METADATA_SIZE (16 * 1024 * 1024)  /* KB */
 
 #define DEFAULT_UMASK 0077
 
