@@ -21,11 +21,6 @@
 
 #include "libdevmapper.h"
 
-/* Define some portable printing types */
-#define PRIsize_t "zu"
-#define PRIptrdiff_t "td"
-#define PRIpid_t PRId32
-
 struct str_list {
 	struct dm_list list;
 	const char *str;
