@@ -19,6 +19,9 @@
 #include "config.h"
 #include "toolcontext.h"
 
+#ifdef UDEV_SYNC_SUPPORT
+#include <libudev.h>
+#endif
 #include <unistd.h>
 #include <sys/param.h>
 #include <dirent.h>
@@ -760,21 +763,44 @@ int dev_cache_init(struct cmd_context *cmd)
 	return 0;
 }
 
-static void _check_closed(struct device *dev)
+/*
+ * Returns number of devices still open.
+ */
+static int _check_for_open_devices(int close_immediate)
 {
-	if (dev->fd >= 0)
-		log_error("Device '%s' has been left open.", dev_name(dev));
+	struct device *dev;
+	struct dm_hash_node *n;
+	int num_open = 0;
+
+	dm_hash_iterate(n, _cache.names) {
+		dev = (struct device *) dm_hash_get_data(_cache.names, n);
+		if (dev->fd >= 0) {
+			log_error("Device '%s' has been left open (%d remaining references).",
+				  dev_name(dev), dev->open_count);
+			num_open++;
+			if (close_immediate)
+				dev_close_immediate(dev);
+		}
+	}
+
+	return num_open;
 }
 
-static void _check_for_open_devices(void)
+/*
+ * Returns number of devices left open.
+ */
+int dev_cache_check_for_open_devices(void)
 {
-	dm_hash_iter(_cache.names, (dm_hash_iterate_fn) _check_closed);
+	return _check_for_open_devices(0);
 }
 
-void dev_cache_exit(void)
+int dev_cache_exit(void)
 {
+	int num_open = 0;
+
 	if (_cache.names)
-		_check_for_open_devices();
+		if ((num_open = _check_for_open_devices(1)) > 0)
+			log_error(INTERNAL_ERROR "%d device(s) were left open and have been closed.", num_open);
 
 	if (_cache.preferred_names_matcher)
 		_cache.preferred_names_matcher = NULL;
@@ -793,6 +819,8 @@ void dev_cache_exit(void)
 	_cache.has_scanned = 0;
 	dm_list_init(&_cache.dirs);
 	dm_list_init(&_cache.files);
+
+	return (!num_open);
 }
 
 int dev_cache_add_dir(const char *path)
