@@ -21,6 +21,10 @@
 #include <libgen.h>
 #include <ctype.h>
 
+#ifdef BLKID_WIPING_SUPPORT
+#include <blkid.h>
+#endif
+
 #include "device-types.h"
 
 struct dev_types *create_dev_types(const char *proc_dir,
@@ -443,15 +447,213 @@ out:
 	return ret;
 }
 
+#ifdef BLKID_WIPING_SUPPORT
+
+static inline int _type_in_flag_list(const char *type, uint32_t flag_list)
+{
+	return (((flag_list & TYPE_LVM2_MEMBER) && !strcmp(type, "LVM2_member")) ||
+		((flag_list & TYPE_LVM1_MEMBER) && !strcmp(type, "LVM1_member")) ||
+		((flag_list & TYPE_DM_SNAPSHOT_COW) && !strcmp(type, "DM_snapshot_cow")));
+}
+
+static int _blkid_wipe(blkid_probe probe, struct device *dev, const char *name,
+		       uint32_t types_to_exclude, uint32_t types_no_prompt,
+		       int yes, force_t force)
+{
+	static const char _msg_failed_offset[] = "Failed to get offset of the %s signature on %s.";
+	static const char _msg_failed_length[] = "Failed to get length of the %s signature on %s.";
+	static const char _msg_wiping[] = "Wiping %s signature on %s.";
+	const char *offset = NULL, *type = NULL, *magic = NULL,
+		   *usage = NULL, *label = NULL, *uuid = NULL;
+	loff_t offset_value;
+	size_t len;
+
+	if (!blkid_probe_lookup_value(probe, "TYPE", &type, NULL)) {
+		if (_type_in_flag_list(type, types_to_exclude))
+			return 1;
+		if (blkid_probe_lookup_value(probe, "SBMAGIC_OFFSET", &offset, NULL)) {
+			log_error(_msg_failed_offset, type, name);
+			return 0;
+		}
+		if (blkid_probe_lookup_value(probe, "SBMAGIC", &magic, &len)) {
+			log_error(_msg_failed_length, type, name);
+			return 0;
+		}
+	} else if (!blkid_probe_lookup_value(probe, "PTTYPE", &type, NULL)) {
+		if (blkid_probe_lookup_value(probe, "PTMAGIC_OFFSET", &offset, NULL)) {
+			log_error(_msg_failed_offset, type, name);
+			return 0;
+		}
+		if (blkid_probe_lookup_value(probe, "PTMAGIC", &magic, &len)) {
+			log_error(_msg_failed_length, type, name);
+			return 0;
+		}
+		usage = "partition table";
+	} else
+		return_0;
+
+	offset_value = strtoll(offset, NULL, 10);
+
+	if (!usage)
+		blkid_probe_lookup_value(probe, "USAGE", &usage, NULL);
+	blkid_probe_lookup_value(probe, "LABEL", &label, NULL);
+	blkid_probe_lookup_value(probe, "UUID", &uuid, NULL);
+
+	log_verbose("Found existing signature on %s at offset %s: LABEL=\"%s\" "
+		    "UUID=\"%s\" TYPE=\"%s\" USAGE=\"%s\"",
+		     name, offset, label, uuid, type, usage);
+
+	if (!_type_in_flag_list(type, types_no_prompt)) {
+		if (!yes && (force == PROMPT) &&
+		    yes_no_prompt("WARNING: %s signature detected on %s at offset %s. "
+				  "Wipe it? [y/n] ", type, name, offset) != 'y')
+			return_0;
+		log_print_unless_silent(_msg_wiping, type, name);
+	} else
+		log_verbose(_msg_wiping, type, name);
+
+	if (!dev_set(dev, offset_value, len, 0)) {
+		log_error("Failed to wipe %s signature on %s.", type, name);
+		return 0;
+	}
+
+	return 1;
+}
+
+static int _wipe_known_signatures_with_blkid(struct device *dev, const char *name,
+					     uint32_t types_to_exclude,
+					     uint32_t types_no_prompt,
+					     int yes, force_t force)
+{
+	blkid_probe probe = NULL;
+	int found = 0, wiped = 0, left = 0;
+	int r = 0;
+
+	/* TODO: Should we check for valid dev - _dev_is_valid(dev)? */
+
+	if (!(probe = blkid_new_probe_from_filename(dev_name(dev)))) {
+		log_error("Failed to create a new blkid probe for device %s.", dev_name(dev));
+		goto out;
+	}
+
+	blkid_probe_enable_partitions(probe, 1);
+	blkid_probe_set_partitions_flags(probe, BLKID_PARTS_MAGIC);
+
+	blkid_probe_enable_superblocks(probe, 1);
+	blkid_probe_set_superblocks_flags(probe, BLKID_SUBLKS_LABEL |
+						 BLKID_SUBLKS_UUID |
+						 BLKID_SUBLKS_TYPE |
+						 BLKID_SUBLKS_USAGE |
+						 BLKID_SUBLKS_VERSION |
+						 BLKID_SUBLKS_MAGIC |
+						 BLKID_SUBLKS_BADCSUM);
+
+	while (!blkid_do_probe(probe)) {
+		found++;
+		if (_blkid_wipe(probe, dev, name, types_to_exclude, types_no_prompt, yes, force))
+			wiped++;
+	}
+
+	if (!found)
+		r = 1;
+
+	left = found - wiped;
+	if (!left)
+		r = 1;
+	else
+		log_warn("%d existing signature%s left on the device.",
+			  left, left > 1 ? "s" : "");
+out:
+	if (probe)
+		blkid_free_probe(probe);
+	return r;
+}
+
+#endif /* BLKID_WIPING_SUPPORT */
+
+static int _wipe_signature(struct device *dev, const char *type, const char *name,
+			   int wipe_len, int yes, force_t force,
+			   int (*signature_detection_fn)(struct device *dev, uint64_t *offset_found))
+{
+	int wipe;
+	uint64_t offset_found;
+
+	wipe = signature_detection_fn(dev, &offset_found);
+	if (wipe == -1) {
+		log_error("Fatal error while trying to detect %s on %s.",
+			  type, name);
+		return 0;
+	}
+
+	if (wipe == 0)
+		return 1;
+
+	/* Specifying --yes => do not ask. */
+	if (!yes && (force == PROMPT) &&
+	    yes_no_prompt("WARNING: %s detected on %s. Wipe it? [y/n] ",
+			  type, name) != 'y')
+		return_0;
+
+	log_print_unless_silent("Wiping %s on %s.", type, name);
+	if (!dev_set(dev, offset_found, wipe_len, 0)) {
+		log_error("Failed to wipe %s on %s.", type, name);
+		return 0;
+	}
+
+	return 1;
+}
+
+static int _wipe_known_signatures_with_lvm(struct device *dev, const char *name,
+					   uint32_t types_to_exclude __attribute__((unused)),
+					   uint32_t types_no_prompt __attribute__((unused)),
+					   int yes, force_t force)
+{
+	if (!_wipe_signature(dev, "software RAID md superblock", name, 4, yes, force, dev_is_md) ||
+	    !_wipe_signature(dev, "swap signature", name, 10, yes, force, dev_is_swap) ||
+	    !_wipe_signature(dev, "LUKS signature", name, 8, yes, force, dev_is_luks))
+		return 0;
+
+	return 1;
+}
+
+int wipe_known_signatures(struct cmd_context *cmd, struct device *dev,
+			  const char *name, uint32_t types_to_exclude,
+			  uint32_t types_no_prompt, int yes, force_t force)
+{
+#ifdef BLKID_WIPING_SUPPORT
+	if (find_config_tree_bool(cmd, allocation_use_blkid_wiping_CFG, NULL))
+		return _wipe_known_signatures_with_blkid(dev, name,
+							 types_to_exclude,
+							 types_no_prompt,
+							 yes, force);
+#endif
+	return _wipe_known_signatures_with_lvm(dev, name,
+					       types_to_exclude,
+					       types_no_prompt,
+					       yes, force);
+}
+
 #ifdef __linux__
 
+static int _snprintf_attr(char *buf, size_t buf_size, const char *sysfs_dir,
+			 const char *attribute, dev_t dev)
+{
+	if (dm_snprintf(buf, buf_size, "%s/dev/block/%d:%d/%s", sysfs_dir,
+			(int)MAJOR(dev), (int)MINOR(dev),
+			attribute) < 0) {
+		log_warn("dm_snprintf %s failed.", attribute);
+		return 0;
+	}
+
+	return 1;
+}
+
 static unsigned long _dev_topology_attribute(struct dev_types *dt,
 					     const char *attribute,
 					     struct device *dev)
 {
 	const char *sysfs_dir = dm_sysfs_dir();
-	static const char sysfs_fmt_str[] = "%s/dev/block/%d:%d/%s";
-	char path[PATH_MAX+1], buffer[64];
+	char path[PATH_MAX], buffer[64];
 	FILE *fp;
 	struct stat info;
 	dev_t uninitialized_var(primary);
@@ -463,12 +665,8 @@ static unsigned long _dev_topology_attribute(struct dev_types *dt,
 	if (!sysfs_dir || !*sysfs_dir)
 		return_0;
 
-	if (dm_snprintf(path, PATH_MAX, sysfs_fmt_str, sysfs_dir,
-			(int)MAJOR(dev->dev), (int)MINOR(dev->dev),
-			attribute) < 0) {
-		log_error("dm_snprintf %s failed", attribute);
-		return 0;
-	}
+	if (!_snprintf_attr(path, sizeof(path), sysfs_dir, attribute, dev->dev))
+                return_0;
 
 	/*
 	 * check if the desired sysfs attribute exists
@@ -484,12 +682,9 @@ static unsigned long _dev_topology_attribute(struct dev_types *dt,
 			return 0;
 
 		/* get attribute from partition's primary device */
-		if (dm_snprintf(path, PATH_MAX, sysfs_fmt_str, sysfs_dir,
-				(int)MAJOR(primary), (int)MINOR(primary),
-				attribute) < 0) {
-			log_error("primary dm_snprintf %s failed", attribute);
-			return 0;
-		}
+		if (!_snprintf_attr(path, sizeof(path), sysfs_dir, attribute, primary))
+			return_0;
+
 		if (stat(path, &info) == -1) {
 			if (errno != ENOENT)
 				log_sys_error("stat", path);
