@@ -17,6 +17,7 @@
 #define _LVM_DEVICE_H
 
 #include "uuid.h"
+#include "lvm-types.h"
 
 #include <fcntl.h>
 
@@ -41,6 +42,7 @@ struct device {
 	int open_count;
 	int error_count;
 	int max_error_count;
+	int phys_block_size;
 	int block_size;
 	int read_ahead;
 	uint32_t flags;
@@ -65,8 +67,8 @@ struct device_area {
 /*
  * All io should use these routines.
  */
+int dev_get_block_size(struct device *dev, unsigned int *phys_block_size, unsigned int *block_size);
 int dev_get_size(const struct device *dev, uint64_t *size);
-int dev_get_sectsize(struct device *dev, uint32_t *size);
 int dev_get_read_ahead(struct device *dev, uint32_t *read_ahead);
 int dev_discard_blocks(struct device *dev, uint64_t offset_bytes, uint64_t size_bytes);
 
@@ -74,6 +76,9 @@ int dev_discard_blocks(struct device *dev, uint64_t offset_bytes, uint64_t size_
 int dev_open(struct device *dev);
 int dev_open_quiet(struct device *dev);
 int dev_open_flags(struct device *dev, int flags, int direct, int quiet);
+int dev_open_readonly(struct device *dev);
+int dev_open_readonly_buffered(struct device *dev);
+int dev_open_readonly_quiet(struct device *dev);
 int dev_close(struct device *dev);
 int dev_close_immediate(struct device *dev);
 void dev_close_all(void);
@@ -96,30 +101,4 @@ struct device *dev_create_file(const char *filename, struct device *dev,
 /* Return a valid device name from the alias list; NULL otherwise */
 const char *dev_name_confirmed(struct device *dev, int quiet);
 
-/* Does device contain md superblock?  If so, where? */
-int dev_is_md(struct device *dev, uint64_t *sb);
-int dev_is_swap(struct device *dev, uint64_t *signature);
-int dev_is_luks(struct device *dev, uint64_t *signature);
-unsigned long dev_md_stripe_width(const char *sysfs_dir, struct device *dev);
-
-int is_partitioned_dev(struct device *dev);
-
-int get_primary_dev(const char *sysfs_dir,
-		    const struct device *dev, dev_t *result);
-
-unsigned long dev_alignment_offset(const char *sysfs_dir,
-				   struct device *dev);
-
-unsigned long dev_minimum_io_size(const char *sysfs_dir,
-				  struct device *dev);
-
-unsigned long dev_optimal_io_size(const char *sysfs_dir,
-				  struct device *dev);
-
-unsigned long dev_discard_max_bytes(const char *sysfs_dir,
-				    struct device *dev);
-
-unsigned long dev_discard_granularity(const char *sysfs_dir,
-				      struct device *dev);
-
 #endif
