@@ -23,7 +23,7 @@
 #include <sys/utsname.h>
 #include <limits.h>
 
-#ifdef linux
+#ifdef __linux__
 #  include "kdev_t.h"
 #  include <linux/limits.h>
 #else
@@ -380,7 +380,8 @@ static int _open_control(void)
 	if (!_uname())
 		return 0;
 
-	snprintf(control, sizeof(control), "%s/%s", dm_dir(), DM_CONTROL_NODE);
+	if (dm_snprintf(control, sizeof(control), "%s/%s", dm_dir(), DM_CONTROL_NODE) < 0)
+		goto_bad;
 
 	/*
 	 * Prior to 2.6.36 the minor number should be looked up in /proc.
@@ -1821,8 +1822,7 @@ int dm_task_run(struct dm_task *dmt)
 	const char *dev_name = DEV_NAME(dmt);
 	const char *dev_uuid = DEV_UUID(dmt);
 
-	if ((unsigned) dmt->type >=
-	    (sizeof(_cmd_data_v4) / sizeof(*_cmd_data_v4))) {
+	if ((unsigned) dmt->type >= DM_ARRAY_SIZE(_cmd_data_v4)) {
 		log_error(INTERNAL_ERROR "unknown device-mapper task %d",
 			  dmt->type);
 		return 0;
