@@ -110,7 +110,7 @@ STACKTRACE() {
 		i=$(($i + 1))
 	done
 
-	test ${LVM_TEST_PARALLEL:-0} -eq 1 -o -n "$RUNNING_DMEVENTD" -o -f LOCAL_DMEVENTD || {
+	test "${LVM_TEST_PARALLEL:-0}" -eq 1 -o -n "$RUNNING_DMEVENTD" -o -f LOCAL_DMEVENTD || {
 		pgrep dmeventd &>/dev/null && \
 			die "** During test dmeventd has been started!"
 	}
@@ -159,7 +159,7 @@ teardown_udev_cookies() {
 }
 
 dm_info() {
-	should dmsetup info --noheadings -c -o "$@"
+	should dmsetup info --mangle none --noheadings -c -o "$@"
 }
 
 dm_table() {
