@@ -27,8 +27,8 @@ lvcreate -l2 -n linear12 $vg "$dev1":4 "$dev2":4
 lvcreate -aey -l1 -n origin1 $vg "$dev1"
 lvcreate -s $vg/origin1 -l1 -n s_napshot2 "$dev2"
 
-lvcreate -aey -l1 -m1 -n mirror12 --mirrorlog core $vg "$dev1" "$dev2"
-lvcreate -aey -l1 -m1 -n mirror123 $vg "$dev1" "$dev2" "$dev3"
+lvcreate -aey -l1 --type mirror -m1 -n mirror12 --mirrorlog core $vg "$dev1" "$dev2"
+lvcreate -aey -l1 --type mirror -m1 -n mirror123 $vg "$dev1" "$dev2" "$dev3"
 
 vgchange -a n $vg
 aux disable_dev "$dev1"
@@ -84,3 +84,5 @@ check active $vg linear2
 check active $vg linear12
 check active $vg mirror12
 check active $vg mirror123
+
+vgremove -ff $vg
