@@ -13,7 +13,7 @@
 
 aux prepare_vg 3
 
-lvcreate -aey -m 1 -l 1 --nosync -n mirror $vg
+lvcreate -aey --type mirror -m 1 -l 1 --nosync -n mirror $vg
 lvchange -a n $vg/mirror
 aux disable_dev "$dev1"
 
@@ -28,4 +28,4 @@ vgchange -a n --partial $vg
 vgchange -aey --partial $vg
 
 # check vgremove
-vgremove -f $vg
+vgremove -ff $vg
