@@ -13,7 +13,7 @@
 
 aux prepare_vg 3 12
 
-lvcreate -aye -m 1 -l 1 -n mirror $vg
+lvcreate -aye --type mirror -m 1 -l 1 -n mirror $vg
 lvcreate -l 1 -n resized $vg
 lvchange -a n $vg/mirror
 
@@ -73,3 +73,5 @@ vgextend $vg "$dev2"
 aux restore_dev "$dev2"
 vgscan $cache
 should check compare_fields vgs $vg vg_mda_count pvs "$dev2" vg_mda_count
+
+vgremove -ff $vg
