@@ -18,74 +18,77 @@ aux lvmconf 'allocation/mirror_logs_require_separate_pvs = 1'
 # Clean-up and create a 2-way mirror, where the the
 # leg devices are always on $dev[12] and the log
 # is always on $dev3.  ($dev4 behaves as a spare)
-cleanup() {
+cleanup_() {
 	vgreduce --removemissing $vg
 	for d in "$@"; do aux enable_dev "$d"; done
 	for d in "$@"; do vgextend $vg "$d"; done
 	lvremove -ff $vg/mirror
-	lvcreate -aey -m 1 --ignoremonitoring -l 2 -n mirror $vg "$dev1" "$dev2" "$dev3":0
+	lvcreate -aey --type mirror -m 1 --ignoremonitoring -l 2 -n mirror $vg "$dev1" "$dev2" "$dev3:0"
 }
 
-repair() {
+repair_() {
 	lvconvert --repair --use-policies --config "$1" $vg/mirror
 }
 
-lvcreate -aey -m 1 -L 1 -n mirror $vg
+lvcreate -aey --type mirror -m 1 --ignoremonitoring -l 2 -n mirror $vg "$dev1" "$dev2" "$dev3:0"
 lvchange -a n $vg/mirror
 
 # Fail a leg of a mirror.
 aux disable_dev "$dev1"
 lvchange --partial -aey $vg/mirror
-repair 'activation { mirror_image_fault_policy = "remove" }'
+repair_ 'activation { mirror_image_fault_policy = "remove" }'
 check linear $vg mirror
-cleanup "$dev1"
+cleanup_ "$dev1"
 
 # Fail a leg of a mirror.
 # Expected result: Mirror (leg replaced, should retain log)
 aux disable_dev "$dev1"
-repair 'activation { mirror_image_fault_policy = "replace" mirror_log_fault_policy = "remove" }'
+repair_ 'activation { mirror_image_fault_policy = "replace" mirror_log_fault_policy = "remove" }'
 check mirror $vg mirror
 check active $vg mirror_mlog
-cleanup "$dev1"
+cleanup_ "$dev1"
 
 # Fail a leg of a mirror.
 # Expected result: Mirror (leg replaced)
 aux disable_dev "$dev1"
-repair 'activation { mirror_image_fault_policy = "replace" }'
+repair_ 'activation { mirror_image_fault_policy = "replace" }'
 check mirror $vg mirror
 check active $vg mirror_mlog
-cleanup "$dev1"
+cleanup_ "$dev1"
 
 # Fail a leg of a mirror (use old name for policy specification)
 # Expected result: Mirror (leg replaced)
 aux disable_dev "$dev1"
-repair 'activation { mirror_image_fault_policy = "replace" }'
+repair_ 'activation { mirror_image_fault_policy = "replace" }'
 check mirror $vg mirror
 check active $vg mirror_mlog
-cleanup "$dev1"
+not pvdisplay $vg
+cleanup_ "$dev1"
 
 # Fail a leg of a mirror w/ no available spare
 # Expected result: linear
 #                  (or 2-way with leg/log overlap if alloc anywhere)
 aux disable_dev "$dev2" "$dev4"
-repair 'activation { mirror_image_fault_policy = "replace" }'
+repair_ 'activation { mirror_image_fault_policy = "replace" }'
 check mirror $vg mirror
-not check lv_exists $vg mirror_mlog
-cleanup "$dev2" "$dev4"
+check lv_not_exists $vg mirror_mlog
+cleanup_ "$dev2" "$dev4"
 
 # Fail the log device of a mirror w/ no available spare
 # Expected result: mirror w/ corelog
 aux disable_dev "$dev3" "$dev4"
-repair 'activation { mirror_image_fault_policy = "replace" }' $vg/mirror
+repair_ 'activation { mirror_image_fault_policy = "replace" }' $vg/mirror
 check mirror $vg mirror
-not check lv_exists $vg mirror_mlog
-cleanup "$dev3" "$dev4"
+check lv_not_exists $vg mirror_mlog
+cleanup_ "$dev3" "$dev4"
 
 # Fail the log device with a remove policy
 # Expected result: mirror w/ corelog
 lvchange -aey $vg/mirror
 aux disable_dev "$dev3" "$dev4"
-repair 'activation { mirror_log_fault_policy = "remove" }'
+repair_ 'activation { mirror_log_fault_policy = "remove" }'
 check mirror $vg mirror core
-not check lv_exists $vg mirror_mlog
-cleanup "$dev3" "$dev4"
+check lv_not_exists $vg mirror_mlog
+cleanup_ "$dev3" "$dev4"
+
+vgremove -ff $vg
