@@ -26,14 +26,14 @@ _check_mlog()
 	not grep mlog out
 }
 
-aux lvmconf 'allocation/maximise_cling = 0'
-aux lvmconf 'allocation/mirror_logs_require_separate_pvs = 1'
+aux lvmconf "allocation/maximise_cling = 0" \
+	    "allocation/mirror_logs_require_separate_pvs = 1"
 
 # fail multiple devices
 
 # 4-way, disk log => 2-way, disk log
 aux prepare_vg 8
-lvcreate -aey -m 3 --ignoremonitoring -L 1 -n 4way $vg "$dev1" "$dev2" "$dev3" "$dev4" "$dev5":0
+lvcreate -aey --type mirror -m 3 --ignoremonitoring -L 1 -n 4way $vg "$dev1" "$dev2" "$dev3" "$dev4" "$dev5":0
 aux disable_dev "$dev2" "$dev4"
 echo n | lvconvert --repair $vg/4way 2>&1 | tee 4way.out
 lvs -a -o +devices $vg | not grep unknown
@@ -43,7 +43,7 @@ check mirror $vg 4way "$dev5"
 
 # 3-way, disk log => linear
 recreate_vg_
-lvcreate -aey -m 2 --ignoremonitoring -L 1 -n 3way $vg
+lvcreate -aey --type mirror -m 2 --ignoremonitoring -L 1 -n 3way $vg
 aux disable_dev "$dev1" "$dev2"
 echo n | lvconvert --repair $vg/3way
 check linear $vg 3way
@@ -56,7 +56,7 @@ check linear $vg 3way
 
 # 3-way, disk log => 3-way, core log
 recreate_vg_
-lvcreate -aey -m 2 --ignoremonitoring -L 1 -n 3way $vg "$dev1" "$dev2" "$dev3" "$dev4":0
+lvcreate -aey --type mirror -m 2 --ignoremonitoring -L 1 -n 3way $vg "$dev1" "$dev2" "$dev3" "$dev4":0
 aux disable_dev "$dev4"
 echo n | lvconvert --repair $vg/3way
 check mirror $vg 3way core
@@ -66,7 +66,7 @@ aux enable_dev "$dev4"
 
 # 3-way, mirrored log => 3-way, core log
 recreate_vg_ -c n
-lvcreate -aey -m 2 --mirrorlog mirrored --ignoremonitoring -L 1 -n 3way $vg \
+lvcreate -aey --type mirror -m 2 --mirrorlog mirrored --ignoremonitoring -L 1 -n 3way $vg \
     "$dev1" "$dev2" "$dev3" "$dev4":0 "$dev5":0
 aux disable_dev "$dev4" "$dev5"
 echo n | lvconvert --repair $vg/3way
@@ -77,7 +77,7 @@ aux enable_dev "$dev4" "$dev5"
 
 # 2-way, disk log => 2-way, core log
 recreate_vg_
-lvcreate -aey -m 1 --ignoremonitoring -L 1 -n 2way $vg "$dev1" "$dev2" "$dev3":0
+lvcreate -aey --type mirror -m 1 --ignoremonitoring -L 1 -n 2way $vg "$dev1" "$dev2" "$dev3":0
 aux disable_dev "$dev3"
 echo n | lvconvert --repair $vg/2way
 check mirror $vg 2way core
@@ -90,7 +90,7 @@ aux enable_dev "$dev3"
 recreate_vg_
 vgreduce $vg "$dev4"
 
-lvcreate -aey -m 1 --ignoremonitoring -L 1 -n mirror $vg
+lvcreate -aey --type mirror -m 1 --ignoremonitoring -L 1 -n mirror $vg
 lvchange -a n $vg/mirror
 vgextend $vg "$dev4"
 aux disable_dev "$dev1"
