@@ -15,11 +15,11 @@ test -e LOCAL_LVMETAD || skip
 
 aux prepare_pvs 2
 
-maj=$(($(stat --printf=0x%t "$dev2")))
-min=$(($(stat --printf=0x%T "$dev2")))
+maj=$(($(stat -L --printf=0x%t "$dev2")))
+min=$(($(stat -L --printf=0x%T "$dev2")))
 
-aux hide_dev $dev2
-not pvscan --cache $dev2 2>&1 | grep "not found"
+aux hide_dev "$dev2"
+not pvscan --cache "$dev2" 2>&1 | grep "not found"
 # pvscan with --major/--minor does not fail: lvmetad needs to
 # be notified about device removal on REMOVE uevent, hence
 # this should not fail so udev does not grab a "failed" state
@@ -27,8 +27,8 @@ not pvscan --cache $dev2 2>&1 | grep "not found"
 # exactly the same command "pvscan --cache" - in case of removal,
 # this is detected by nonexistence of the device itself.
 pvscan --cache --major $maj --minor $min 2>&1 | grep "not found"
-aux unhide_dev $dev2
+aux unhide_dev "$dev2"
 
-pvscan --cache $dev2 2>&1 | not grep "not found"
+pvscan --cache "$dev2" 2>&1 | not grep "not found"
 pvscan --cache --major $maj --minor $min 2>&1 | not grep "not found"
-pvs | grep $dev2
+pvs | grep "$dev2"
