@@ -17,33 +17,35 @@ rm LOCAL_LVMETAD
 
 aux prepare_devs 2
 
-pvcreate --metadatacopies 0 $dev1
-pvcreate --metadatacopies 1 $dev2
-vgcreate $vg1 $dev1 $dev2
+pvcreate --metadatacopies 0 "$dev1"
+pvcreate --metadatacopies 1 "$dev2"
+vgcreate $vg1 "$dev1" "$dev2"
 lvcreate -n foo -l 1 -an --zero n $vg1
 
 # start lvmetad but make sure it doesn't know about $dev1 or $dev2
-aux disable_dev $dev1
-aux disable_dev $dev2
+aux disable_dev "$dev1"
+aux disable_dev "$dev2"
 aux prepare_lvmetad
 lvs
 mv LOCAL_LVMETAD XXX
-aux enable_dev $dev2
-aux enable_dev $dev1
+aux enable_dev "$dev2"
+aux enable_dev "$dev1"
 mv XXX LOCAL_LVMETAD
 
 aux lvmconf 'global/use_lvmetad = 0'
 check inactive $vg1 foo
 aux lvmconf 'global/use_lvmetad = 1'
 
-pvscan --cache $dev2 -aay
+pvscan --cache "$dev2" -aay
 
 aux lvmconf 'global/use_lvmetad = 0'
 check inactive $vg1 foo
 aux lvmconf 'global/use_lvmetad = 1'
 
-pvscan --cache $dev1 -aay
+pvscan --cache "$dev1" -aay
 
 aux lvmconf 'global/use_lvmetad = 0'
 check active $vg1 foo
 aux lvmconf 'global/use_lvmetad = 1'
+
+vgremove -ff $vg1
