@@ -15,12 +15,12 @@ aux prepare_devs 3
 
 vgcreate --metadatasize 128k $vg1 "$dev1" "$dev2" "$dev3"
 
-vgreduce $vg1 $dev1
+vgreduce $vg1 "$dev1"
 dd if="$dev1" of=badmda bs=256K count=1
-vgextend $vg1 $dev1
+vgextend $vg1 "$dev1"
 
 dd if=badmda of="$dev1" bs=256K count=1
 
 # dev1 is part of vg1 (as witnessed by metadata on dev2 and dev3), but its mda
 # was corrupt (written over by a backup from time dev1 was an orphan)
-check pv_field $dev1 vg_name $vg1
+check pv_field "$dev1" vg_name $vg1
