@@ -25,7 +25,7 @@ lv_devices_() {
 	local devs=$(get lv_field $lv devices -a | sed 's/([0-9]*)//g; s/ //g; s/,/ /g')
 
 	for d in $devs; do
-		(echo $devices | grep $d) || return 1
+		(echo "$devices" | grep $d) || return 1
 		devices=$(echo $devices | sed "s/$d//")
 	done
 
@@ -63,7 +63,7 @@ check_and_cleanup_lvs_
 #COMM "init: lvcreate"
 
 #COMM "mirror images are ${lv1}_mimage_x"
-lvcreate -an -Zn -l2 -m1 -n $lv1 $vg
+lvcreate -an -Zn -l2 --type mirror -m1 -n $lv1 $vg
 lv_devices_ $vg/$lv1 ${lv1}_mimage_0 ${lv1}_mimage_1
 
 #COMM "mirror log is ${lv1}_mlog"
@@ -74,7 +74,7 @@ check_and_cleanup_lvs_
 
 #COMM "mirror with name longer than 22 characters (bz221322)"
 name="LVwithanamelogerthan22characters_butidontwonttocounthem"
-lvcreate -an -Zn -m1 -l2 -n $name $vg
+lvcreate -an -Zn --type mirror -m1 -l2 -n $name $vg
 lvs $vg/$name
 check_and_cleanup_lvs_
 
@@ -84,7 +84,7 @@ check_and_cleanup_lvs_
 #COMM "init: lvrename"
 
 #COMM "renamed mirror names: $lv1 to $lv2"
-lvcreate -an -Zn -l2 -m1 -n $lv1 $vg
+lvcreate -an -Zn -l2 --type mirror -m1 -n $lv1 $vg
 lvrename $vg/$lv1 $vg/$lv2
 lv_devices_ $vg/$lv2 ${lv2}_mimage_0 ${lv2}_mimage_1
 lv_mirror_log_ $vg/$lv2 ${lv2}_mlog
@@ -98,7 +98,7 @@ check_and_cleanup_lvs_
 #COMM "init: lvconvert"
 
 #COMM "converting mirror names is ${lv1}_mimagetmp_2"
-lvcreate -aey -l2 -m1 -n $lv1 $vg
+lvcreate -aey -l2 --type mirror -m1 -n $lv1 $vg
 lvconvert -m+1 -i+40 -b $vg/$lv1
 convlv=$(lv_convert_lv_ $vg/$lv1)
 test $convlv = ${lv1}_mimagetmp_2
