@@ -21,8 +21,11 @@ do
 	if mke2fs "$dev1"; then
 		mount "$dev1" mnt
 		not pvcreate -M$mdatype "$dev1" 2>err
-		grep "Can't open "$dev1" exclusively.  Mounted filesystem?" err
+		grep "Can't open $dev1 exclusively.  Mounted filesystem?" err
 		umount "$dev1"
+		# wipe the filesystem signature for next
+		# pvcreate to not issue any prompts
+		dd if=/dev/zero of="$dev1" bs=1K count=2
 	fi
 
 # pvcreate (lvm$mdatype) succeeds when run repeatedly (pv not in a vg) (bz178216)
@@ -113,13 +116,29 @@ not pvcreate --uuid $uuid2 --restorefile $backupfile "$dev2"
 
 # vgcfgrestore of a VG containing a PV with zero PEs (bz #820116)
 # (use case: one PV in a VG used solely to keep metadata)
-size_mb=$(($(blockdev --getsz $dev1) / 2048))
-pvcreate --metadatasize $size_mb $dev1
-vgcreate $vg1 $dev1
+size_mb=$(($(blockdev --getsz "$dev1") / 2048))
+pvcreate --metadatasize $size_mb "$dev1"
+vgcreate $vg1 "$dev1"
 vgcfgbackup -f $backupfile
 vgcfgrestore -f $backupfile $vg1
 vgremove -f $vg1
-pvremove -f $dev1
+pvremove -f "$dev1"
+
+# pvcreate --restorefile should handle --dataalignment and --dataalignmentoffset
+# and check it's compatible with pe_start value being restored
+# X * dataalignment + dataalignmentoffset == pe_start
+pvcreate --norestorefile --uuid $uuid1 --dataalignment 600k --dataalignmentoffset 32k "$dev1"
+vgcreate $vg1 "$dev1"
+vgcfgbackup -f $backupfile $vg1
+vgremove -ff $vg1
+pvremove -ff "$dev1"
+# the dataalignment and dataalignmentoffset is ignored here since they're incompatible with pe_start
+pvcreate --restorefile $backupfile --uuid $uuid1 --dataalignment 500k --dataalignmentoffset 10k "$dev1" 2> err
+grep "incompatible with restored pe_start value" err
+# 300k is multiple of 600k so this should pass
+pvcreate --restorefile $backupfile --uui $uuid1 --dataalignment 300k --dataalignmentoffset 32k "$dev1" 2> err
+not grep "incompatible with restored pe_start value" err
+rm -f $backupfile
 
 # pvcreate rejects non-existent uuid given with restorefile
 not pvcreate --uuid $uuid1 --restorefile $backupfile "$dev1"
