@@ -20,7 +20,7 @@ check_() {
 
 aux prepare_vg 3
 
-lvcreate -an -Zn -m 1 -l 1 -n mirror $vg
+lvcreate -an -Zn --type mirror -m 1 -l 1 -n mirror $vg
 #lvchange -a n $vg
 
 # try orphaning a missing PV (bz45867)
@@ -42,3 +42,5 @@ aux enable_dev "$dev1"
 check_
 test -e LOCAL_LVMETAD && lvremove $vg/boo # FIXME trigger a write :-(
 check_ not
+
+vgremove -ff $vg
