@@ -1,5 +1,5 @@
 #!/bin/sh
-# Copyright (C) 2010-2011 Red Hat, Inc. All rights reserved.
+# Copyright (C) 2010-2014 Red Hat, Inc. All rights reserved.
 #
 # This copyrighted material is made available to anyone wishing to use,
 # modify, copy, or redistribute it subject to the terms and conditions
@@ -16,18 +16,42 @@ aux prepare_devs 2
 vgcreate --metadatasize 128k $vg1 "$dev1"
 lvcreate -l100%FREE -n $lv1 $vg1
 
+# Test plain vgexport vgimport tools
+
+# Argument is needed
+invalid vgexport
+invalid vgimport
+# Cannot combine -a and VG name
+invalid vgexport -a $vg
+invalid vgimport -a $vg1
+# Cannot export unknonw VG
+fail vgexport ${vg1}-non
+fail vgimport ${vg1}-non
+# Cannot export VG with active volumes
+fail vgexport $vg1
+
+vgchange -an $vg1
+vgexport $vg1
+# Already exported
+fail vgexport $vg1
+
+vgimport $vg1
+# Already imported
+fail vgimport $vg1
+vgchange -ay $vg1
+
 # Clone the LUN
 dd if="$dev1" of="$dev2" bs=256K count=1
 aux notify_lvmetad "$dev2"
 
 # Verify pvs works on each device to give us vgname
-aux hide_dev $dev2
+aux hide_dev "$dev2"
 check pv_field "$dev1" vg_name $vg1
-aux unhide_dev $dev2
+aux unhide_dev "$dev2"
 
-aux hide_dev $dev1
+aux hide_dev "$dev1"
 check pv_field "$dev2" vg_name $vg1
-aux unhide_dev $dev1
+aux unhide_dev "$dev1"
 
 # Import the cloned PV to a new VG
 vgimportclone --basevgname $vg2 "$dev2"
@@ -43,3 +67,5 @@ aux notify_lvmetad "$dev1"
 # Verify we can activate / deactivate the LV from both VGs
 lvchange -ay $vg1/$lv1 $vg2/$lv1
 vgchange -an $vg1 $vg2
+
+vgremove -ff $vg1 $vg2
