@@ -906,6 +906,8 @@ static int _get_settings(struct cmd_context *cmd)
 	else
 		init_ignorelockingfailure(0);
 
+	cmd->ignore_clustered_vgs = arg_count(cmd, ignoreskippedcluster_ARG) ? 1 : 0;
+
 	if (!arg_count(cmd, sysinit_ARG))
 		lvmetad_connect_or_warn();
 
@@ -1135,6 +1137,8 @@ int lvm_run_command(struct cmd_context *cmd, int argc, char **argv)
 			log_error("Failed to apply configuration profile.");
 			return ECMD_FAILED;
 		}
+		if (!process_profilable_config(cmd))
+			return_ECMD_FAILED;
 	}
 
 	if ((ret = _get_settings(cmd)))
@@ -1181,7 +1185,7 @@ int lvm_run_command(struct cmd_context *cmd, int argc, char **argv)
       out:
 	if (test_mode()) {
 		log_verbose("Test mode: Wiping internal cache");
-		lvmcache_destroy(cmd, 1);
+		lvmcache_destroy(cmd, 1, 0);
 	}
 
 	if ((old_cft = remove_config_tree_by_source(cmd, CONFIG_STRING))) {
