@@ -36,18 +36,18 @@ static int _pvdisplay_single(struct cmd_context *cmd,
 			return ret;
 		}
 
-	 	/*
+		/*
 		 * Replace possibly incomplete PV structure with new one
 		 * allocated in vg_read_internal() path.
 		 */
-		 if (!(pvl = find_pv_in_vg(vg, pv_name))) {
-			 log_error("Unable to find \"%s\" in volume group \"%s\"",
-				   pv_name, vg->name);
-			 ret = ECMD_FAILED;
-			 goto out;
-		 }
+		if (!(pvl = find_pv_in_vg(vg, pv_name))) {
+			log_error("Unable to find \"%s\" in volume group \"%s\"",
+				  pv_name, vg->name);
+			ret = ECMD_FAILED;
+			goto out;
+		}
 
-		 pv = pvl->pv;
+		pv = pvl->pv;
 	}
 
 	if (is_orphan(pv))
@@ -63,12 +63,12 @@ static int _pvdisplay_single(struct cmd_context *cmd,
 	}
 
 	if (pv_status(pv) & EXPORTED_VG)
-		log_print("Physical volume \"%s\" of volume group \"%s\" "
-			  "is exported", pv_name, pv_vg_name(pv));
+		log_print_unless_silent("Physical volume \"%s\" of volume group \"%s\" "
+					"is exported", pv_name, pv_vg_name(pv));
 
 	if (is_orphan(pv))
-		log_print("\"%s\" is a new physical volume of \"%s\"",
-			  pv_name, display_size(cmd, size));
+		log_print_unless_silent("\"%s\" is a new physical volume of \"%s\"",
+					pv_name, display_size(cmd, size));
 
 	if (arg_count(cmd, colon_ARG)) {
 		pvdisplay_colons(pv);
@@ -98,18 +98,26 @@ int pvdisplay(struct cmd_context *cmd, int argc, char **argv)
 			return EINVALID_CMD_LINE;
 		}
 		return pvs(cmd, argc, argv);
-	} else if (arg_count(cmd, aligned_ARG) ||
-		   arg_count(cmd, all_ARG) ||
-		   arg_count(cmd, noheadings_ARG) ||
-		   arg_count(cmd, options_ARG) ||
-		   arg_count(cmd, separator_ARG) ||
-		   arg_count(cmd, sort_ARG) || arg_count(cmd, unbuffered_ARG)) {
+	}
+
+	if (arg_count(cmd, aligned_ARG) ||
+	    arg_count(cmd, all_ARG) ||
+	    arg_count(cmd, noheadings_ARG) ||
+	    arg_count(cmd, options_ARG) ||
+	    arg_count(cmd, separator_ARG) ||
+	    arg_count(cmd, sort_ARG) ||
+	    arg_count(cmd, unbuffered_ARG)) {
 		log_error("Incompatible options selected");
 		return EINVALID_CMD_LINE;
 	}
 
 	if (arg_count(cmd, colon_ARG) && arg_count(cmd, maps_ARG)) {
-		log_error("Option -v not allowed with option -c");
+		log_error("Option -c not allowed with option -m");
+		return EINVALID_CMD_LINE;
+	}
+
+	if (arg_count(cmd, colon_ARG) && arg_count(cmd, short_ARG)) {
+		log_error("Option -c is not allowed with option -s");
 		return EINVALID_CMD_LINE;
 	}
 
