@@ -18,23 +18,23 @@
 #include "lvmetad.h"
 #include "lvmcache.h"
 
-int pv_max_name_len = 0;
-int vg_max_name_len = 0;
+unsigned pv_max_name_len = 0;
+unsigned vg_max_name_len = 0;
 
 static void _pvscan_display_single(struct cmd_context *cmd,
 				   struct physical_volume *pv,
 				   void *handle __attribute__((unused)))
 {
-	char uuid[64] __attribute__((aligned(8)));
-	unsigned vg_name_len = 0;
-
-	char pv_tmp_name[NAME_LEN] = { 0 };
-	char vg_tmp_name[NAME_LEN] = { 0 };
-	char vg_name_this[NAME_LEN] = { 0 };
+	/* XXXXXX-XXXX-XXXX-XXXX-XXXX-XXXX-XXXXXX */
+	char uuid[40] __attribute__((aligned(8)));
+	const unsigned suffix = sizeof(uuid) + 10;
+	char pv_tmp_name[pv_max_name_len + suffix];
+	unsigned pv_len = pv_max_name_len;
+	const char *pvdevname = pv_dev_name(pv);
 
 	/* short listing? */
 	if (arg_count(cmd, short_ARG) > 0) {
-		log_print_unless_silent("%s", pv_dev_name(pv));
+		log_print_unless_silent("%s", pvdevname);
 		return;
 	}
 
@@ -49,59 +49,52 @@ static void _pvscan_display_single(struct cmd_context *cmd,
 		/* return; */
 	}
 
-	vg_name_len = strlen(pv_vg_name(pv)) + 1;
-
 	if (arg_count(cmd, uuid_ARG)) {
 		if (!id_write_format(&pv->id, uuid, sizeof(uuid))) {
 			stack;
 			return;
 		}
 
-		sprintf(pv_tmp_name, "%-*s with UUID %s",
-			pv_max_name_len - 2, pv_dev_name(pv), uuid);
-	} else {
-		sprintf(pv_tmp_name, "%s", pv_dev_name(pv));
+		if (dm_snprintf(pv_tmp_name, sizeof(pv_tmp_name), "%-*s with UUID %s",
+				pv_max_name_len - 2, pvdevname, uuid) < 0) {
+			log_error("Invalid PV name with uuid.");
+			return;
+		}
+		pvdevname = pv_tmp_name;
+		pv_len += suffix;
 	}
 
-	if (is_orphan(pv)) {
+	if (is_orphan(pv))
 		log_print_unless_silent("PV %-*s    %-*s %s [%s]",
-					pv_max_name_len, pv_tmp_name,
+					pv_len, pvdevname,
 					vg_max_name_len, " ",
 					pv->fmt ? pv->fmt->name : "    ",
 					display_size(cmd, pv_size(pv)));
-		return;
-	}
-
-	if (pv_status(pv) & EXPORTED_VG) {
-		strncpy(vg_name_this, pv_vg_name(pv), vg_name_len);
-		log_print_unless_silent("PV %-*s  is in exported VG %s "
-					"[%s / %s free]",
-					pv_max_name_len, pv_tmp_name,
-					vg_name_this,
+	else if (pv_status(pv) & EXPORTED_VG)
+		log_print_unless_silent("PV %-*s  is in exported VG %s [%s / %s free]",
+					pv_len, pvdevname, pv_vg_name(pv),
+					display_size(cmd, (uint64_t) pv_pe_count(pv) * pv_pe_size(pv)),
+					display_size(cmd, (uint64_t) (pv_pe_count(pv) - pv_pe_alloc_count(pv)) * pv_pe_size(pv)));
+	else
+		log_print_unless_silent("PV %-*s VG %-*s %s [%s / %s free]",
+					pv_len, pvdevname,
+					vg_max_name_len, pv_vg_name(pv),
+					pv->fmt ? pv->fmt->name : "    ",
 					display_size(cmd, (uint64_t) pv_pe_count(pv) * pv_pe_size(pv)),
 					display_size(cmd, (uint64_t) (pv_pe_count(pv) - pv_pe_alloc_count(pv)) * pv_pe_size(pv)));
-		return;
-	}
-
-	sprintf(vg_tmp_name, "%s", pv_vg_name(pv));
-	log_print_unless_silent("PV %-*s VG %-*s %s [%s / %s free]", pv_max_name_len,
-				pv_tmp_name, vg_max_name_len, vg_tmp_name,
-				pv->fmt ? pv->fmt->name : "    ",
-				display_size(cmd, (uint64_t) pv_pe_count(pv) * pv_pe_size(pv)),
-				display_size(cmd, (uint64_t) (pv_pe_count(pv) - pv_pe_alloc_count(pv)) * pv_pe_size(pv)));
 }
 
 #define REFRESH_BEFORE_AUTOACTIVATION_RETRIES 5
 #define REFRESH_BEFORE_AUTOACTIVATION_RETRY_USLEEP_DELAY 100000
 
 static int _auto_activation_handler(struct cmd_context *cmd,
-				    const char *vgid, int partial,
+				    const char *vgname, const char *vgid,
+				    int partial, int changed,
 				    activation_change_t activate)
 {
 	unsigned int refresh_retries = REFRESH_BEFORE_AUTOACTIVATION_RETRIES;
 	int refresh_done = 0;
 	struct volume_group *vg;
-	int consistent = 0;
 	struct id vgid_raw;
 	int r = 0;
 
@@ -113,8 +106,12 @@ static int _auto_activation_handler(struct cmd_context *cmd,
 		return_0;
 
 	/* NB. This is safe because we know lvmetad is running and we won't hit disk. */
-	if (!(vg = vg_read_internal(cmd, NULL, (const char *) &vgid_raw, 0, &consistent)))
-	    return 1;
+	vg = vg_read(cmd, vgname, (const char *)&vgid_raw, 0);
+	if (vg_read_error(vg)) {
+		log_error("Failed to read Volume Group \"%s\" (%s) during autoactivation.", vgname, vgid);
+		release_vg(vg);
+		return 0;
+	}
 
 	if (vg_is_clustered(vg)) {
 		r = 1; goto out;
@@ -139,16 +136,18 @@ static int _auto_activation_handler(struct cmd_context *cmd,
 	 *
 	 * Remove this workaround with "refresh_retries" once we have proper locking in!
 	 */
-	while (refresh_retries--) {
-		if (vg_refresh_visible(vg->cmd, vg)) {
-			refresh_done = 1;
-			break;
+	if (changed) {
+		while (refresh_retries--) {
+			if (vg_refresh_visible(vg->cmd, vg)) {
+				refresh_done = 1;
+				break;
+			}
+			usleep(REFRESH_BEFORE_AUTOACTIVATION_RETRY_USLEEP_DELAY);
 		}
-		usleep(REFRESH_BEFORE_AUTOACTIVATION_RETRY_USLEEP_DELAY);
-	}
 
-	if (!refresh_done)
-		log_warn("%s: refresh before autoactivation failed.", vg->name);
+		if (!refresh_done)
+			log_warn("%s: refresh before autoactivation failed.", vg->name);
+	}
 
 	if (!vgchange_activate(vg->cmd, vg, activate)) {
 		log_error("%s: autoactivation failed.", vg->name);
@@ -158,7 +157,7 @@ static int _auto_activation_handler(struct cmd_context *cmd,
 	r = 1;
 
 out:
-	release_vg(vg);
+	unlock_and_release_vg(cmd, vg, vgname);
 	return r;
 }
 
@@ -170,15 +169,13 @@ static int _clear_dev_from_lvmetad_cache(dev_t devno, int32_t major, int32_t min
 	if (!dm_asprintf(&buf, "%" PRIi32 ":%" PRIi32, major, minor))
 		stack;
 	if (!lvmetad_pv_gone(devno, buf ? : "", handler)) {
-		if (buf)
-			dm_free(buf);
+		dm_free(buf);
 		return 0;
 	}
 
 	log_print_unless_silent("Device %s not found. "
 				"Cleared from lvmetad cache.", buf ? : "");
-	if (buf)
-		dm_free(buf);
+	dm_free(buf);
 
 	return 1;
 }
@@ -332,20 +329,22 @@ int pvscan(struct cmd_context *cmd, int argc, char **argv)
 
 	uint64_t size_total = 0;
 	uint64_t size_new = 0;
-
-	int len = 0;
-	pv_max_name_len = 0;
-	vg_max_name_len = 0;
+	unsigned len;
 
 	if (arg_count(cmd, cache_ARG))
 		return _pvscan_lvmetad(cmd, argc, argv);
 
+	if (argc) {
+		log_error("Too many parameters on command line.");
+		return EINVALID_CMD_LINE;
+	}
+
 	if (arg_count(cmd, activate_ARG)) {
 		log_error("--activate is only valid with --cache.");
 		return EINVALID_CMD_LINE;
 	}
 
-	if (arg_count(cmd, major_ARG) + arg_count(cmd, minor_ARG)) {
+	if (arg_count(cmd, major_ARG) || arg_count(cmd, minor_ARG)) {
 		log_error("--major and --minor are only valid with --cache.");
 		return EINVALID_CMD_LINE;
 	}
@@ -367,7 +366,7 @@ int pvscan(struct cmd_context *cmd, int argc, char **argv)
 
 	if (cmd->filter->wipe)
 		cmd->filter->wipe(cmd->filter);
-	lvmcache_destroy(cmd, 1);
+	lvmcache_destroy(cmd, 1, 0);
 
 	/* populate lvmcache */
 	if (!lvmetad_vg_list_to_lvmcache(cmd))
@@ -430,18 +429,15 @@ int pvscan(struct cmd_context *cmd, int argc, char **argv)
 		free_pv_fid(pvl->pv);
 	}
 
-	if (!pvs_found) {
+	if (!pvs_found)
 		log_print_unless_silent("No matching physical volumes found");
-		unlock_vg(cmd, VG_GLOBAL);
-		return ECMD_PROCESSED;
-	}
-
-	log_print_unless_silent("Total: %d [%s] / in use: %d [%s] / in no VG: %d [%s]",
-				pvs_found,
-				display_size(cmd, size_total),
-				pvs_found - new_pvs_found,
-				display_size(cmd, (size_total - size_new)),
-				new_pvs_found, display_size(cmd, size_new));
+	else
+		log_print_unless_silent("Total: %d [%s] / in use: %d [%s] / in no VG: %d [%s]",
+					pvs_found,
+					display_size(cmd, size_total),
+					pvs_found - new_pvs_found,
+					display_size(cmd, (size_total - size_new)),
+					new_pvs_found, display_size(cmd, size_new));
 
 	unlock_vg(cmd, VG_GLOBAL);
 
