@@ -83,6 +83,8 @@ static int vg_rename_path(struct cmd_context *cmd, const char *old_vg_path,
 	if (!lvmetad_vg_list_to_lvmcache(cmd))
 		stack;
 
+	lvmcache_label_scan(cmd, 2);
+
 	/* Avoid duplicates */
 	if (!(vgids = get_vgids(cmd, 0)) || dm_list_empty(vgids)) {
 		log_error("No complete volume groups found");
@@ -183,7 +185,7 @@ static int vg_rename_path(struct cmd_context *cmd, const char *old_vg_path,
 	/* FIXME lvmcache corruption - vginfo duplicated instead of renamed */
 	if (cmd->filter->wipe)
 		cmd->filter->wipe(cmd->filter);
-	lvmcache_destroy(cmd, 1);
+	lvmcache_destroy(cmd, 1, 0);
 
 	return 1;
 
@@ -205,10 +207,8 @@ int vgrename(struct cmd_context *cmd, int argc, char **argv)
 		return EINVALID_CMD_LINE;
 	}
 
-	if (!vg_rename_path(cmd, argv[0], argv[1])) {
-		stack;
-		return ECMD_FAILED;
-	}
+	if (!vg_rename_path(cmd, argv[0], argv[1]))
+		return_ECMD_FAILED;
 
 	return ECMD_PROCESSED;
 }
